/*************************************************************************
 * Copyright (c) 2004 Altera Corporation, San Jose, California, USA.      *
 * All rights reserved. All use of this software and documentation is     *
 * subject to the License Agreement located at the end of this file below.*
 **************************************************************************
 * Description:                                                           *
 * The following is a simple hello world program running MicroC/OS-II.The *
 * purpose of the design is to be a very simple application that just     *
 * demonstrates MicroC/OS-II running on NIOS II.The design doesn't account*
 * for issues such as checking system call return codes. etc.             *
 *                                                                        *
 * Requirements:                                                          *
 *   -Supported Example Hardware Platforms                                *
 *     Standard                                                           *
 *     Full Featured                                                      *
 *     Low Cost                                                           *
 *   -Supported Development Boards                                        *
 *     Nios II Development Board, Stratix II Edition                      *
 *     Nios Development Board, Stratix Professional Edition               *
 *     Nios Development Board, Stratix Edition                            *
 *     Nios Development Board, Cyclone Edition                            *
 *   -System Library Settings                                             *
 *     RTOS Type - MicroC/OS-II                                           *
 *     Periodic System Timer                                              *
 *   -Know Issues                                                         *
 *     If this design is run on the ISS, terminal output will take several*
 *     minutes per iteration.                                             *
 **************************************************************************/


#include <stdio.h>
#include "includes.h"
#include "altera_avalon_pio_regs.h"
#include "alt_types.h"
//random
#include <stdlib.h>
#include <stdint.h>

#include <math.h>

/* Definition of Task Stacks */
#define   TASK_STACKSIZE       2048
OS_STK    write_data_stk[TASK_STACKSIZE];
OS_STK    read_data_stk[TASK_STACKSIZE];

/* Definition of Task Priorities */

#define WRITE_PRIORITY 1
#define READ_PRIORITY 2

#define DE2_FREQ 50000000
#define HALF_STEP 2
#define ORIGINAL_STEP 0X7fffffff

#define AVERAGE 700
#define DEV 200
#define PI 3.14159265358979

//generate random frequency for stepper mortor
int get_random_freq(){
	int freq;
	int iRandom1, iRandom2;
	double dRandom1, dRandom2;
	//generates a number between 0 and 4G
	iRandom1=(rand());
	//offset to have unifor ditribution between -2G and +2G
	dRandom1=(double)iRandom1 /RAND_MAX;

	//generates a number between 0 and 4G
	iRandom2=(rand());
	//offset to have unifor ditribution between -2G and +2G
	dRandom2=(double)iRandom2 /RAND_MAX;

	//Output random values. The formula is based on Box-Muller method
	freq = AVERAGE + DEV * sqrt(-2.0 * log(dRandom1))*cos(2 * PI * dRandom2);

	printf("random freq is %d\n",freq);
	return freq;
}

//generate random direction for stepper motor
int get_random_direction(){
	int direction;
	//generate random number between[0,4]
	direction = rand()%4;
	return direction;
}

//write data into specific address
void write_data(void* pdata)
{
	int frequency = (DE2_FREQ/get_random_freq())/2-1;
	while (1)
	{
		IOWR_ALTERA_AVALON_PIO_DATA(FREQ_DIVIDER_INTERFACE_0_BASE, frequency);
		IOWR_ALTERA_AVALON_PIO_DATA(DIR_STOP_INTERFACE_0_BASE, get_random_direction());

		OSTimeDlyHMSM(0, 0, 3, 0);
	}
}
//read data from specific address
void read_data(void* pdata)
{
	int current_step;
	while (1)
	{
		printf("frequency: %d\n",IORD_ALTERA_AVALON_PIO_DATA(FREQ_DIVIDER_INTERFACE_0_BASE));
		printf("direction: %d\n",IORD_ALTERA_AVALON_PIO_DATA(DIR_STOP_INTERFACE_0_BASE));
		current_step = IORD_ALTERA_AVALON_PIO_DATA(MOTOR_CONTROLLER_0_BASE)-ORIGINAL_STEP;
		printf("current step: %d\n",current_step);
		OSTimeDlyHMSM(0, 0, 3, 0);
	}
}
/* The main function creates two task and starts multi-tasking */
int main(void)
{

	OSTaskCreateExt(write_data,
			NULL,
			(void *)&write_data_stk[TASK_STACKSIZE-1],
			WRITE_PRIORITY,
			WRITE_PRIORITY,
			write_data_stk,
			TASK_STACKSIZE,
			NULL,
			0);


	OSTaskCreateExt(read_data,
			NULL,
			(void *)&read_data_stk[TASK_STACKSIZE-1],
			READ_PRIORITY,
			READ_PRIORITY,
			read_data_stk,
			TASK_STACKSIZE,
			NULL,
			0);
	OSStart();
	return 0;
}

/******************************************************************************
 *                                                                             *
 * License Agreement                                                           *
 *                                                                             *
 * Copyright (c) 2004 Altera Corporation, San Jose, California, USA.           *
 * All rights reserved.                                                        *
 *                                                                             *
 * Permission is hereby granted, free of charge, to any person obtaining a     *
 * copy of this software and associated documentation files (the "Software"),  *
 * to deal in the Software without restriction, including without limitation   *
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,    *
 * and/or sell copies of the Software, and to permit persons to whom the       *
 * Software is furnished to do so, subject to the following conditions:        *
 *                                                                             *
 * The above copyright notice and this permission notice shall be included in  *
 * all copies or substantial portions of the Software.                         *
 *                                                                             *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR  *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,    *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER      *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING     *
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER         *
 * DEALINGS IN THE SOFTWARE.                                                   *
 *                                                                             *
 * This agreement shall be governed in all respects by the laws of the State   *
 * of California and by the laws of the United States of America.              *
 * Altera does not recommend, suggest or require that this reference design    *
 * file be used in conjunction or combination with any other product.          *
 ******************************************************************************/
