/*************************************************************************
 * Copyright (c) 2004 Altera Corporation, San Jose, California, USA.      *
 * All rights reserved. All use of this software and documentation is     *
 * subject to the License Agreement located at the end of this file below.*
 **************************************************************************
 * Description:                                                           *
 * The following is a simple hello world program running MicroC/OS-II.The *
 * purpose of the design is to be a very simple application that just     *
 * demonstrates MicroC/OS-II running on NIOS II.The design doesn't account*
 * for issues such as checking system call return codes. etc.             *
 *                                                                        *
 * Requirements:                                                          *
 *   -Supported Example Hardware Platforms                                *
 *     Standard                                                           *
 *     Full Featured                                                      *
 *     Low Cost                                                           *
 *   -Supported Development Boards                                        *
 *     Nios II Development Board, Stratix II Edition                      *
 *     Nios Development Board, Stratix Professional Edition               *
 *     Nios Development Board, Stratix Edition                            *
 *     Nios Development Board, Cyclone Edition                            *
 *   -System Library Settings                                             *
 *     RTOS Type - MicroC/OS-II                                           *
 *     Periodic System Timer                                              *
 *   -Know Issues                                                         *
 *     If this design is run on the ISS, terminal output will take several*
 *     minutes per iteration.                                             *
 **************************************************************************/

#include <stdio.h>
#include "includes.h"
#include "system.h"
#include "altera_avalon_pio_regs.h"
#include "alt_types.h"
#include "altera_up_avalon_character_lcd_regs.h"
#include "alt_sys_init.c"

/*generate random number*/
//random
#include <stdlib.h>
#include <stdint.h>

#include <string.h>
//for bool
#include <stdbool.h>
//for pow
#include <math.h>
//sleep
#include <unistd.h>
#include <time.h>
//////////////////////////

/* Definition of Task Stacks */
#define   TASK_STACKSIZE       2048
OS_STK    write_data_stk[TASK_STACKSIZE];
OS_STK    read_data_stk[TASK_STACKSIZE];
OS_STK	read_gun_ID_stk[TASK_STACKSIZE];
OS_STK	read_gun_ID_1_stk[TASK_STACKSIZE];
OS_STK	read_key_stk[TASK_STACKSIZE];

/* Definition of Task Priorities */
#define WRITE_PRIORITY 3
#define READ_PRIORITY 4
#define GUN_ID_PRIORITY 2
#define GUN_ID_PRIORITY_1 5
#define KEY_PRIORITY 1
//used to calculate frequency to send to vhd code
#define DE2_FREQ 50000000
#define HALF_STEP 2
//used for gaussian random number
#define AVERAGE 700
#define DEV 200
#define PI 3.14159265358979
//information of stepper motor
#define RADIUS 2
#define STEPS_PER_ROTATION 8
//the original step is set in vhd code
#define ORIGINAL_STEP 0X7fffffff
//safe height and width
#define WIDTH 120
#define HEIGHT 100

//the structure to record steps for motor
struct Motor{
	int curr_steps;
	int old_steps;
};
//the structure to record the information of position
struct Point{
	double curr_x;
	double curr_y;
	double old_x;
	double old_y;
	double cos_angle_left;
	double cos_angle_right;
	double line_left;
	double line_right;
	int direction_left;
	int direction_right;
};
/*-----------------------game-----------------------*/
/* messages for ISR routine */
#define N_RESOURCES			4
OS_EVENT *QSem;
OS_EVENT *KEY_QSem;
void *QMsg[N_RESOURCES];
void *KEY_QMsg[N_RESOURCES];

INT8U err = OS_NO_ERR;

// stores edge capture of IR
volatile int edge_capture;

#ifdef IR_RUPT_INTERFACE_BASE

#ifdef ALT_ENHANCED_INTERRUPT_API_PRESENT
static void handle_IR_interrupts(void* context)
#else
static void handle_IR_interrupts(void* context, alt_u32 id)
#endif
{
	/* Cast context to edge_capture's type. It is important that this be
	 * declared volatile to avoid unwanted compiler optimization.
	 */
	volatile int* edge_capture_ptr = (volatile int*) context;
	/* Store the value in the IR edge capture register in *context. */
	*edge_capture_ptr = IORD_ALTERA_AVALON_PIO_EDGE_CAP(IR_RUPT_INTERFACE_BASE);
	/* Reset the IR edge capture register. */
	IOWR_ALTERA_AVALON_PIO_EDGE_CAP(IR_RUPT_INTERFACE_BASE, 0);
	// transfer register data to task
	alt_u32 content = * edge_capture_ptr;
	OSQPost(QSem, (void*) content);
	// small delay before exiting the interrupt
	IORD_ALTERA_AVALON_PIO_EDGE_CAP(IR_RUPT_INTERFACE_BASE);
}

static void init_IR_pio()
{
	/* Recast the edge_capture pointer to match the alt_irq_register() function
	 * prototype. */
	void* edge_capture_ptr = (void*) &edge_capture;
	/* Enable all 8 bit interrupts for edge capture register. */
	IOWR_ALTERA_AVALON_PIO_IRQ_MASK(IR_RUPT_INTERFACE_BASE, 0xff);
	/* Reset the edge capture register. */
	IOWR_ALTERA_AVALON_PIO_EDGE_CAP(IR_RUPT_INTERFACE_BASE, 0x0);
	/* Register the interrupt handler. */
#ifdef ALT_ENHANCED_INTERRUPT_API_PRESENT
	alt_ic_isr_register(IR_RUPT_INTERFACE_IRQ_INTERRUPT_CONTROLLER_ID, IR_RUPT_INTERFACE_IRQ,
			handle_IR_interrupts, edge_capture_ptr, 0x0);
#else
	alt_irq_register( IR_RUPT_INTERFACE_IRQ, edge_capture_ptr,
			handle_IR_interrupts);
#endif
}
#endif

//record the information of IDs
struct GUN {
	int count;
};
//check the hitted times of each ID
struct GUN check_count(struct GUN myID0, struct GUN myID1){
	if (myID0.count > myID1.count){
		return myID0;
	}else{
		return myID1;
	}
}

/*---------------------------------------------------*/

struct Point my_point = {100,100,0,0,0,0,0,0,0,0};

//generate random frequency for stepper motor
int get_random_freq(){
	int freq;
	int iRandom1, iRandom2;
	double dRandom1, dRandom2;
	//generates a number between 0 and 4G
	iRandom1=(rand());
	//offset to have unifor ditribution between -2G and +2G
	dRandom1=(double)iRandom1 /RAND_MAX;

	//generates a number between 0 and 4G
	iRandom2=(rand());
	//offset to have unifor ditribution between -2G and +2G
	dRandom2=(double)iRandom2 /RAND_MAX;

	//Output random values. The formula is based on Box-Muller method
	freq = AVERAGE + DEV * sqrt(-2.0 * log(dRandom1))*cos(2 * PI * dRandom2);

	//printf("random freq is %d\n",freq);
	return freq;
}

//generate random direction for stepper motor
int get_random_direction(){
	int direction;
	//generate random number between[0,4]
	direction = rand()%4;
	//reduce the occurrence of 0 and 2 to reduce the possibility of stop
	if (direction== 0||direction==2){
		direction = rand()%4;
	}
	return direction;
}

//calculate the value of cos(left_angle)
struct Point cos_angle(struct Point my_point){
	my_point.cos_angle_left = abs(pow(my_point.line_right,2)-pow(WIDTH,2)-pow(my_point.line_left,2))/(2*my_point.line_left*WIDTH);
	my_point.cos_angle_right = abs(pow(my_point.line_left,2)-pow(WIDTH,2)-pow(my_point.line_right,2))/(2*my_point.line_right*WIDTH);
	return my_point;
}

//calculate the ideal position of receiver
struct Point calculate_position(struct Point my_point){
	double sin_left = 0;
	my_point.curr_x = my_point.cos_angle_left * my_point.line_left;
	sin_left = sqrt(1 - pow(my_point.cos_angle_left,2));
	my_point.curr_y = my_point.line_left * sin_left;
	return my_point;
}

//test the position of receiver
int test_position(struct Point my_point){
	if ((WIDTH - my_point.curr_x) <= 10){
		return 0;
	}else{
		if ((WIDTH - my_point.curr_x) >= 90){
			return 1;
		}else{
			if ((HEIGHT - my_point.curr_y) <= 10){
				return 2;
			}else{
				if ((HEIGHT - my_point.curr_y) >= 90){
					return 3;
				}else{
					return 4;
				}
			}
		}
	}
	return 0;
}

//write data to specific address
void write_data(void* pdata)
{
	//calculate the frequency from random number
	int frequ0 = (DE2_FREQ/get_random_freq())/2-1;
	int frequ1 = (DE2_FREQ/get_random_freq())/2-1;

	while (1)
	{
		//write frequency to 2 motors separately
		IOWR_ALTERA_AVALON_PIO_DATA(FREQ_DIVIDER_INTERFACE_0_BASE, frequ0);
		IOWR_ALTERA_AVALON_PIO_DATA(FREQ_DIVIDER_INTERFACE_1_BASE, frequ1);
		//write direction to 2 motors separately
		IOWR_ALTERA_AVALON_PIO_DATA(DIR_STOP_INTERFACE_0_BASE, my_point.direction_left);
		IOWR_ALTERA_AVALON_PIO_DATA(DIR_STOP_INTERFACE_1_BASE, my_point.direction_right);
		OSTimeDlyHMSM(0, 0, 1, 0);
	}
}
//read data from specific address
void read_data(void* pdata)
{
	//initial structure
	struct Motor motor0 = {0,0};
	struct Motor motor1 = {0,0};

	while (1)
	{
		//show the data that has written into specific address
		printf("freq0: %d\n",IORD_ALTERA_AVALON_PIO_DATA(FREQ_DIVIDER_INTERFACE_0_BASE));
		printf("freq1: %d\n",IORD_ALTERA_AVALON_PIO_DATA(FREQ_DIVIDER_INTERFACE_1_BASE));
		printf("direction0: %d\n",IORD_ALTERA_AVALON_PIO_DATA(DIR_STOP_INTERFACE_0_BASE));
		printf("direction1: %d\n",IORD_ALTERA_AVALON_PIO_DATA(DIR_STOP_INTERFACE_1_BASE));

		//calculate the net steps the motor has rotated
		motor0.curr_steps = IORD_ALTERA_AVALON_PIO_DATA(MOTOR_CONTROLLER_0_BASE)-ORIGINAL_STEP;
		motor1.curr_steps = IORD_ALTERA_AVALON_PIO_DATA(MOTOR_CONTROLLER_1_BASE)-ORIGINAL_STEP;

		printf("step0: %d\n",motor0.curr_steps);
		printf("step1: %d\n",motor1.curr_steps);
		//calculate the length of lines
		my_point.line_left =  abs((18/3600)*(2*PI*RADIUS)*motor0.curr_steps);
		my_point.line_right = abs((18/3600)*(2*PI*RADIUS)*motor1.curr_steps);
		my_point = cos_angle(my_point);
		my_point = calculate_position(my_point);

		//show the reaction to the test result
		if(test_position(my_point) == 0){
			my_point.direction_left = 0;
			my_point.direction_right = get_random_direction();
		}
		if(test_position(my_point) == 1){
			my_point.direction_left = get_random_direction();
			my_point.direction_right = 2;
		}
		if(test_position(my_point) == 2){
			my_point.direction_left = 0;
			my_point.direction_right = 2;
		}
		if(test_position(my_point) == 3){
			my_point.direction_left = 2;
			my_point.direction_right = 0;
		}
		if(test_position(my_point) == 4){
			my_point.direction_left = get_random_direction();
			my_point.direction_right = get_random_direction();
		}

		OSTimeDlyHMSM(0, 0, 1, 0);

	}
}

//game mode 1
void read_gun_ID(){
	alt_u32 ID_number;
	struct GUN GUN0 = {0};
	struct GUN GUN1 = {0};
	char buffer[10];

	//initial the lcd
	alt_up_character_lcd_dev * lcd;
	lcd = alt_up_character_lcd_open_dev(CHARACTER_LCD_0_NAME);
	alt_up_character_lcd_init(lcd);
	alt_up_character_lcd_set_cursor_pos(lcd, 0, 0);
	alt_up_character_lcd_string(lcd,"WINER IS:");
	alt_up_character_lcd_set_cursor_pos(lcd, 0, 1);
	while (1){
		//receive the ID number
		ID_number = (alt_u32)OSQPend (QSem,0,&err);
		printf("ID number is %lx\n",ID_number);
		//receiver receive signal from two guns
		if (ID_number == 255){
			GUN0.count++;
			GUN1.count++;
		}
		//receiver receive signal from one gun
		if (ID_number == 240){
			GUN0.count++;
		}
		//receiver receive signal from another gun
		if (ID_number == 15){
			GUN1.count++;
		}

		printf("The count of ID 0 is %d\n",GUN0.count);
		printf("The count of ID 1 is %d\n",GUN1.count);

		if (GUN0.count >= 2 || GUN1.count >= 2){
			snprintf (buffer, 10, "%lx",ID_number);
			alt_up_character_lcd_string(lcd,buffer);
			alt_up_character_lcd_set_cursor_pos(lcd, 0, 1);

		}
		OSTimeDlyHMSM(0, 0, 1, 0);

	}
}
//game mode 2
void read_gun_ID_1(){
	alt_u32 ID_number;
	struct GUN GUN0 = {0};
	char buffer[10];

	//Initial lcd
	alt_up_character_lcd_dev * lcd;
	lcd = alt_up_character_lcd_open_dev(CHARACTER_LCD_0_NAME);
	alt_up_character_lcd_init(lcd);
	alt_up_character_lcd_set_cursor_pos(lcd, 0, 0);
	alt_up_character_lcd_string(lcd,"SHOOTING:");
	alt_up_character_lcd_set_cursor_pos(lcd, 0, 1);

	while (1){
		ID_number = (alt_u32)OSQPend (QSem,0,&err);
		printf("ID number is %lx\n",ID_number);
		//record the times of hit
		if (ID_number == 240 || ID_number == 15){
			GUN0.count++;
		}
		printf("The count of ID %lx is %d\n",ID_number,GUN0.count);
			snprintf (buffer, 10, "%d",GUN0.count);

			alt_up_character_lcd_string(lcd,buffer);
			alt_up_character_lcd_set_cursor_pos(lcd, 0, 1);
		OSTimeDlyHMSM(0, 0, 1, 0);

	}
}

volatile int key_edge_capture;

#ifdef KEY_RUPT_BASE

#ifdef ALT_ENHANCED_INTERRUPT_API_PRESENT
static void handle_KEY_interrupts(void* key_context)
#else
static void handle_KEY_interrupts(void* key_context, alt_u32 key_id)
#endif
{
	/* Cast context to edge_capture's type. It is important that this be
	 * declared volatile to avoid unwanted compiler optimization.
	 */
	volatile int* key_edge_capture_ptr = (volatile int*) key_context;
	/* Store the value in the IR edge capture register in *context. */
	*key_edge_capture_ptr = IORD_ALTERA_AVALON_PIO_EDGE_CAP(KEY_RUPT_BASE);
	/* Reset the IR edge capture register. */
	IOWR_ALTERA_AVALON_PIO_EDGE_CAP(KEY_RUPT_BASE, 0);
	// transfer register data to task
	alt_u32 key_content = * key_edge_capture_ptr;
	OSQPost(KEY_QSem, (void*) key_content);
	// small delay before exiting the interrupt
	IORD_ALTERA_AVALON_PIO_EDGE_CAP(KEY_RUPT_BASE);
}

static void init_KEY_pio()
{
	/* Recast the edge_capture pointer to match the alt_irq_register() function
	 * prototype. */
	void* key_edge_capture_ptr = (void*) &key_edge_capture;
	/* Enable all 8 bit interrupts for edge capture register. */
	IOWR_ALTERA_AVALON_PIO_IRQ_MASK(KEY_RUPT_BASE, 0xff);
	/* Reset the edge capture register. */
	IOWR_ALTERA_AVALON_PIO_EDGE_CAP(KEY_RUPT_BASE, 0x0);
	/* Register the interrupt handler. */
#ifdef ALT_ENHANCED_INTERRUPT_API_PRESENT
	alt_ic_isr_register(KEY_RUPT_IRQ_INTERRUPT_CONTROLLER_ID, KEY_RUPT_IRQ,
			handle_KEY_interrupts, key_edge_capture_ptr, 0x0);
#else
	alt_key_register( KEY_RUPT_IRQ, key_edge_capture_ptr,
			handle_KEY_interrupts);
#endif
}
#endif

//read the data from buttons
void read_key(){
	int key_ID;
	while(1){
		key_ID = (int)OSQPend (KEY_QSem,0,&err);
		//start the game mode 1 immediately
		if (key_ID == 1){
			OSTaskCreateExt(write_data,
					NULL,
					(void *)&write_data_stk[TASK_STACKSIZE-1],
					WRITE_PRIORITY,
					WRITE_PRIORITY,
					write_data_stk,
					TASK_STACKSIZE,
					NULL,
					0);

			OSTaskCreateExt(read_data,
					NULL,
					(void *)&read_data_stk[TASK_STACKSIZE-1],
					READ_PRIORITY,
					READ_PRIORITY,
					read_data_stk,
					TASK_STACKSIZE,
					NULL,
					0);
			OSTaskCreateExt(read_gun_ID,
					NULL,
					(void *)&read_gun_ID_stk[TASK_STACKSIZE-1],
					GUN_ID_PRIORITY,
					GUN_ID_PRIORITY,
					read_gun_ID_stk,
					TASK_STACKSIZE,
					NULL,
					0);
		}
		//start the game with mode 1 after 10 seconds
		if (key_ID == 2){
			OSTimeDlyHMSM(0, 0, 10, 0);
			OSTaskCreateExt(write_data,
					NULL,
					(void *)&write_data_stk[TASK_STACKSIZE-1],
					WRITE_PRIORITY,
					WRITE_PRIORITY,
					write_data_stk,
					TASK_STACKSIZE,
					NULL,
					0);

			OSTaskCreateExt(read_data,
					NULL,
					(void *)&read_data_stk[TASK_STACKSIZE-1],
					READ_PRIORITY,
					READ_PRIORITY,
					read_data_stk,
					TASK_STACKSIZE,
					NULL,
					0);
			OSTaskCreateExt(read_gun_ID,
					NULL,
					(void *)&read_gun_ID_stk[TASK_STACKSIZE-1],
					GUN_ID_PRIORITY,
					GUN_ID_PRIORITY,
					read_gun_ID_stk,
					TASK_STACKSIZE,
					NULL,
					0);
		}
		//change the game mode to mode 2
		if (key_ID == 4){

			OSTaskCreateExt(write_data,
					NULL,
					(void *)&write_data_stk[TASK_STACKSIZE-1],
					WRITE_PRIORITY,
					WRITE_PRIORITY,
					write_data_stk,
					TASK_STACKSIZE,
					NULL,
					0);

			OSTaskCreateExt(read_data,
					NULL,
					(void *)&read_data_stk[TASK_STACKSIZE-1],
					READ_PRIORITY,
					READ_PRIORITY,
					read_data_stk,
					TASK_STACKSIZE,
					NULL,
					0);
			OSTaskCreateExt(read_gun_ID_1,
					NULL,
					(void *)&read_gun_ID_1_stk[TASK_STACKSIZE-1],
					GUN_ID_PRIORITY_1,
					GUN_ID_PRIORITY_1,
					read_gun_ID_1_stk,
					TASK_STACKSIZE,
					NULL,
					0);
		}
		OSTimeDlyHMSM(0, 0, 1, 0);

	}
}
/* The main function creates two task and starts multi-tasking */
int main(void)
{
#ifdef IR_RUPT_INTERFACE_BASE
	init_IR_pio();
#endif

#ifdef KEY_RUPT_BASE
	init_KEY_pio();
#endif
	/* create OS message queue */
	QSem = OSQCreate(&QMsg[0], N_RESOURCES);
	KEY_QSem = OSQCreate(&KEY_QMsg[0], N_RESOURCES);

	OSTaskCreateExt(read_key,
			NULL,
			(void *)&read_key_stk[TASK_STACKSIZE-1],
			KEY_PRIORITY,
			KEY_PRIORITY,
			read_key_stk,
			TASK_STACKSIZE,
			NULL,
			0);
	OSStart();

	return 0;
}

/******************************************************************************
 *                                                                             *
 * License Agreement                                                           *
 *                                                                             *
 * Copyright (c) 2004 Altera Corporation, San Jose, California, USA.           *
 * All rights reserved.                                                        *
 *                                                                             *
 * Permission is hereby granted, free of charge, to any person obtaining a     *
 * copy of this software and associated documentation files (the "Software"),  *
 * to deal in the Software without restriction, including without limitation   *
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,    *
 * and/or sell copies of the Software, and to permit persons to whom the       *
 * Software is furnished to do so, subject to the following conditions:        *
 *                                                                             *
 * The above copyright notice and this permission notice shall be included in  *
 * all copies or substantial portions of the Software.                         *
 *                                                                             *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR  *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,    *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER      *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING     *
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER         *
 * DEALINGS IN THE SOFTWARE.                                                   *
 *                                                                             *
 * This agreement shall be governed in all respects by the laws of the State   *
 * of California and by the laws of the United States of America.              *
 * Altera does not recommend, suggest or require that this reference design    *
 * file be used in conjunction or combination with any other product.          *
 ******************************************************************************/
